package cn.sh.stc.sict.theme.hphy.schedule;

import cn.hutool.core.collection.CollectionUtil;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HpHosInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDeptInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpHosInfoService;
import cn.sh.stc.sict.theme.hphy.wd.DeptInfo;
import cn.sh.stc.sict.theme.hphy.wd.DoctInfo;
import cn.sh.stc.sict.theme.hphy.wd.HosInfo;
import cn.sh.stc.sict.theme.hphy.wd.WanDaHttpUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 定时任务
 * 1. 每月更新医院信息
 * 2. 每周更新科室信息（一级科室，二级科室）
 * 3. 每周更新医生信息
 * 4. 每天更新可预约信息
 */
@Slf4j
@Component
@AllArgsConstructor
public class RefreshJob {

    private final HpHosInfoService hpHosInfoService;
    private final HpDeptInfoService deptInfoService;
    private final HpDocInfoService docInfoService;

    private static final byte TOP_DEPT = 1;
    private static final byte TWO_DEPT = 2;

    @Scheduled(cron = "0 44 13 * * ?")
    public void updateHosp(){
        HosInfo hosInfo = new HosInfo();
        List<HosInfo> list = WanDaHttpUtil.getHospitalInfo(hosInfo);

        if (CollectionUtil.isEmpty(list)){
            return;
        }

        MapperFactory factory = new DefaultMapperFactory.Builder().build();
        List<HpHosInfo> hpHosInfos = factory.getMapperFacade().mapAsList(list, HpHosInfo.class);

        for (HpHosInfo hpHosInfo : hpHosInfos) {
            saveOrUpdateHospital(hpHosInfo);
        }
    }

    private void saveOrUpdateHospital(HpHosInfo hpHosInfo){
        try {
            if (hpHosInfoService.hospitalExist(hpHosInfo.getHosOrgCode())){
                hpHosInfoService.updateById(hpHosInfo);
            }else {
                hpHosInfoService.save(hpHosInfo);
            }
        }catch (Exception e){
            log.error("医院保存出错：{}", hpHosInfo);
            log.error(e.getMessage(), e);
        }
    }

//    @Scheduled(cron = "")
    public void updateDept(){
        List<HpHosInfo> hospitals = hpHosInfoService.list();
        if (CollectionUtil.isEmpty(hospitals)){
            return;
        }

        // TODO 获取一级科室
        MapperFactory factory = new DefaultMapperFactory.Builder().build();
        for (HpHosInfo hospital : hospitals) {
            saveOrUpdateTopDept(hospital.getHosOrgCode(), factory);
        }

        // TODO 获取二级科室

    }

    private void saveOrUpdateTopDept(String hosOrgCode, MapperFactory factory){
        List<DeptInfo> deptInfos = WanDaHttpUtil.getDeptInfoTop(new HosInfo(hosOrgCode));
        if (CollectionUtil.isEmpty(deptInfos)){
            return;
        }

        List<HpDeptInfo> hpDeptInfos = cloneTopDept(deptInfos);


    }



//    @Scheduled(cron = "")
    public void updateDeptTwo(){

    }

    @Scheduled(cron = "0 42 14 * * ?")
    public void updateDocInfo(){
        List<HpHosInfo> list = hpHosInfoService.list();

        if (CollectionUtil.isEmpty(list)){
            return;
        }

        MapperFactory factory = new DefaultMapperFactory.Builder().build();
        for (HpHosInfo hpHosInfo : list) {
            saveOrUpdateDoctor(hpHosInfo.getHosOrgCode(), factory);
        }

    }

    private void saveOrUpdateDoctor(String hosOrgCode, MapperFactory factory){
        List<DoctInfo> doctInfos = WanDaHttpUtil.getDoctInfo(new DeptInfo(hosOrgCode));

        if (CollectionUtil.isEmpty(doctInfos)){
            return;
        }

        List<HpDocInfo> docInfos = factory.getMapperFacade().mapAsList(doctInfos, HpDocInfo.class);

        for (HpDocInfo docInfo : docInfos) {
            saveOrUpdateDoctor(docInfo);
        }
    }

    private void saveOrUpdateDoctor(HpDocInfo docInfo){
        try {
            if (docInfoService.doctorExist(docInfo.getResourceCode())){
                docInfoService.updateById(docInfo);
            }else {
                docInfoService.save(docInfo);
            }
        }catch (Exception e){
            log.error("医生信息保存出错：{}", docInfo);
        }
    }

    private List<HpDeptInfo> cloneTopDept(List<DeptInfo> deptInfos){
        List<HpDeptInfo> hpDeptInfos = new ArrayList<>();
        for (DeptInfo deptInfo : deptInfos) {
            HpDeptInfo hpDeptInfo = new HpDeptInfo();
            hpDeptInfo.setHosOrgCode(deptInfo.getHosOrgCode());
            hpDeptInfo.setOneDeptCode(deptInfo.getDeptCode());
            hpDeptInfo.setHosName(deptInfo.getHosName());
            hpDeptInfo.setOneDeptName(deptInfo.getDeptName());
            hpDeptInfo.setNormdeptCode(deptInfo.getNormdeptCode());
            hpDeptInfo.setNormdeptName(deptInfo.getNormdeptName());
            hpDeptInfo.setDeptDesc(deptInfo.getDeptDesc());
            hpDeptInfo.setDeptLevel(TOP_DEPT);
            hpDeptInfo.setIndexNo(deptInfo.getIndexNo());

            hpDeptInfos.add(hpDeptInfo);
        }

        return hpDeptInfos;
    }

}
